@@ -34,7 +34,7 @@
 #include "vstr.h"
 #include "config.h"
 
-static char gp_title[] = "dvdspanky 1.0.3 - (C) 2007 Jeffrey Grembecki\n\n";
+static char gp_title[] = "dvdspanky 1.0.5 - (C) 2007 Jeffrey Grembecki\n\n";
 static char gp_licence[] =
 "This program is free software; you can redistribute it and/or modify\n"
 "it under the terms of the GNU General Public License as published by\n"
@@ -141,81 +141,102 @@ static pcre *gp_tcaudioregex = NULL;
 static int g_msglevel = ML_INFO;
 
 static FILE *gp_logfile = NULL;
+static vstr *gp_str = NULL;
 
 static void print(int msglevel, const char *p_format, ...)
 {
-	static vstr *p_str = NULL;
+	char *p_dyn;
+	int length;
 	va_list argp;
 
-	if(!p_str)
-	{
-		if(!(p_str = vs_new()))
-			return;
-	}
+	p_dyn = NULL;
 
-	if(msglevel >= g_msglevel)
+	va_start(argp, p_format);
+	length = vsnprintf(NULL, 0, p_format, argp);
+	va_end(argp);
+
+	if(length && (p_dyn = malloc(length + 1)))
 	{
 		va_start(argp, p_format);
-		vs_vprint(p_str, p_format, argp);
+		vsprintf(p_dyn, p_format, argp);
 		va_end(argp);
-		printf("%s", p_str->s);
-	}
 
-	if(gp_logfile)
-	{
-		vs_print(p_str, " dvdspanky | ");
-		va_start(argp, p_format);
-		vs_vcat(p_str, p_format, argp);
-		va_end(argp);
-		fwrite(p_str->s, 1, p_str->len, gp_logfile);
-		fflush(gp_logfile);
+		if(msglevel >= g_msglevel)
+			fprintf(stderr, "%s", p_dyn);
+
+		if(gp_logfile)
+		{
+			vs_print(gp_str, " dvdspanky | %s", p_dyn);
+			fwrite(gp_str->s, 1, gp_str->len, gp_logfile);
+			fflush(gp_logfile);
+		}
+
+		free(p_dyn);
 	}
 }
 
 static void pprint(int msglevel, const char *p_prefix, const char *p_format, ...)
 {
-	static vstr *p_str = NULL;
+	char *p_dyn;
+	int length;
 	va_list argp;
 
-	if(!p_str)
-	{
-		if(!(p_str = vs_new()))
-			return;
-	}
+	p_dyn = NULL;
 
-	vs_print(p_str, "%10s | ", p_prefix);
 	va_start(argp, p_format);
-	vs_vcat(p_str, p_format, argp);
+	length = vsnprintf(NULL, 0, p_format, argp);
 	va_end(argp);
 
-	if(msglevel >= g_msglevel)
-		printf("%s", p_str->s);
-
-	if(gp_logfile)
+	if(length && (p_dyn = malloc(length + 1)))
 	{
-		fwrite(p_str->s, 1, p_str->len, gp_logfile);
-		fflush(gp_logfile);
+		va_start(argp, p_format);
+		vsprintf(p_dyn, p_format, argp);
+		va_end(argp);
+
+		if(msglevel >= g_msglevel)
+			fprintf(stderr, "%s", p_dyn);
+
+		if(gp_logfile)
+		{
+			vs_print(gp_str, "%10s | %s", p_prefix, p_dyn);
+			fwrite(gp_str->s, 1, gp_str->len, gp_logfile);
+			fflush(gp_logfile);
+		}
+
+		free(p_dyn);
 	}
 }
 
 static void printerror(const char *p_format, ...)
 {
-	static vstr *p_str = NULL;
+	char *p_dyn;
+	int length;
 	va_list argp;
 
-	if(!p_str)
-	{
-		if(!(p_str = vs_new()))
-			return;
-	}
+	p_dyn = NULL;
 
-	vs_print(p_str, "error: ");
 	va_start(argp, p_format);
-	vs_vcat(p_str, p_format, argp);
+	length = vsnprintf(NULL, 0, p_format, argp);
 	va_end(argp);
-	vs_cat(p_str, "\n");
 
-	print(ML_ERROR, p_str->s);
+	if(length && (p_dyn = malloc(length + 1)))
+	{
+		va_start(argp, p_format);
+		vsprintf(p_dyn, p_format, argp);
+		va_end(argp);
+
+		if(ML_ERROR >= g_msglevel)
+			fprintf(stderr, "%s", p_dyn);
+
+		if(gp_logfile)
+		{
+			vs_print(gp_str, " dvdspanky | %s", p_dyn);
+			fwrite(gp_str->s, 1, gp_str->len, gp_logfile);
+			fflush(gp_logfile);
+		}
+
+		free(p_dyn);
+	}
 }
 
 /* termination cleanup */
@@ -600,19 +621,19 @@ int cb_tcprobe(const char *p_string, void *p_options)
 
 		/* video settings found, update any missing */
 		if(!p_video->fps)
-			p_video->fps = atof(&p_buffer[p_vector[2]]);
+			p_video->fps = atof(&(p_buffer[p_vector[2]]));
 
 		if(!p_video->p_vcodec[0])
-			strncpy(p_video->p_vcodec, &p_buffer[p_vector[4]], sizeof(p_video->p_vcodec));
+			strncpy(p_video->p_vcodec, &(p_buffer[p_vector[4]]), sizeof(p_video->p_vcodec));
 
 		if(!p_video->frames)
-			p_video->frames = atoi(&p_buffer[p_vector[6]]);
+			p_video->frames = atoi(&(p_buffer[p_vector[6]]));
 
 		if(!p_video->width)
-			p_video->width = atoi(&p_buffer[p_vector[8]]);
+			p_video->width = atoi(&(p_buffer[p_vector[8]]));
 
 		if(!p_video->height)
-			p_video->width = atoi(&p_buffer[p_vector[10]]);
+			p_video->width = atoi(&(p_buffer[p_vector[10]]));
 
 		if(p_video->width && p_video->height)
 			p_video->aspect = (double)p_video->width / (double)p_video->height;
@@ -640,16 +661,16 @@ int cb_tcprobe(const char *p_string, void *p_options)
 
 		/* audio settings found, update any missing */
 		if(!p_video->hz)
-			p_video->hz = atoi(&p_buffer[p_vector[2]]);
+			p_video->hz = atoi(&(p_buffer[p_vector[2]]));
 
 		if(!p_video->abits)
-			p_video->abits = atoi(&p_buffer[p_vector[4]]);
+			p_video->abits = atoi(&(p_buffer[p_vector[4]]));
 
 		if(!p_video->achans)
-			p_video->achans = atoi(&p_buffer[p_vector[6]]);
+			p_video->achans = atoi(&(p_buffer[p_vector[6]]));
 
 		if(!p_video->arate)
-			p_video->arate = atoi(&p_buffer[p_vector[8]]);
+			p_video->arate = atoi(&(p_buffer[p_vector[8]]));
 
 		free(p_buffer);
 
@@ -691,12 +712,12 @@ int cb_transcode(const char *p_string, void *p_options)
 		for(i = 1; i < 6; i++)
 			p_buffer[p_vector[i * 2 + 1]] = 0;
 		if(p_video->frames)
-			fprintf(stderr,"  %6.2f fps, %.2f%% complete, frame %d  \r", atof(&p_buffer[p_vector[4]]),
-				atof(&p_buffer[p_vector[2]]) * 100.0 / (double)p_video->frames, atoi(&p_buffer[p_vector[2]]));
+			fprintf(stderr,"  %6.2f fps, %.2f%% complete, frame %d  \r", atof(&(p_buffer[p_vector[4]])),
+				atof(&(p_buffer[p_vector[2]])) * 100.0 / (double)p_video->frames, atoi(&(p_buffer[p_vector[2]])));
 		else
-			fprintf(stderr,"  %6.2f fps, frame %09d, %d:%02d:%02d  \r", atof(&p_buffer[p_vector[4]]),
-				atoi(&p_buffer[2]), atoi(&p_buffer[p_vector[4]]), atoi(&p_buffer[p_vector[6]]),
-				atoi(&p_buffer[p_vector[8]]));
+			fprintf(stderr,"  %6.2f fps, frame %09d, %d:%02d:%02d  \r", atof(&(p_buffer[p_vector[4]])),
+				atoi(&(p_buffer[2])), atoi(&(p_buffer[p_vector[4]])), atoi(&(p_buffer[p_vector[6]])),
+				atoi(&(p_buffer[p_vector[8]])));
 	}
 	else
 	{
@@ -861,7 +882,7 @@ int forkit(forkitcallback callback, void *p_options, const char *p_path, const c
 			start = 0;
 			while(waitpid(pid, &status, WNOHANG) != pid)
 			{
-				count = read(p_pipe[0], &p_buffer[start], sizeof(p_buffer) - 1 - start);
+				count = read(p_pipe[0], &(p_buffer[start]), sizeof(p_buffer) - 1 - start);
 				if(count)
 				{
 					/* seperate lines and send each to callback */
@@ -872,7 +893,7 @@ int forkit(forkitcallback callback, void *p_options, const char *p_path, const c
 						{
 							/* send line to callback */
 							p_buffer[pos] = 0;
-							if(callback && callback(&p_buffer[start], p_options))
+							if(callback && callback(&(p_buffer[start]), p_options))
 								retval = -3;
 							start = pos + 1;
 						}
@@ -933,7 +954,13 @@ int main(int argc, char **argv)
 	int options = 0;
 	video_t invid, outvid;
 
+	/* setup */
 	p_filters[0] = 0;
+	if(!(gp_str = vs_new()))
+	{
+		fprintf(stderr, "unable to allocate gp_str\n");
+		exit(1);
+	}
 
 	/* clear video data */
 	memset(&invid, '\0', sizeof(video_t));
@@ -1119,7 +1146,6 @@ int main(int argc, char **argv)
 		invid.bytes = sdata.st_size;
 	}
 
-
 	/* create output filename based on input filename if none given */
 	if(!outvid.p_filename[0])
 	{
@@ -1137,7 +1163,7 @@ int main(int argc, char **argv)
 			p_vector, sizeof(p_vector) / sizeof(int));
 		if(found > 0)
 		{
-			strncpy(outvid.p_filename, &invid.p_filename[p_vector[2]], p_vector[3] - p_vector[2]);
+			strncpy(outvid.p_filename, &(invid.p_filename[p_vector[2]]), p_vector[3] - p_vector[2]);
 			outvid.p_filename[p_vector[3] - p_vector[2]] = 0;
 		}
 		else
